<?php

namespace Btravel\Infobjects\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Infobjects\Http\Requests\InfobjectRequest;
use Btravel\Infobjects\Interfaces\InfobjectRepositoryInterface;
use Btravel\Infobjects\Models\Infobject;

/**
 * Resource controller class for infobject.
 */
class InfobjectResourceController extends BaseController
{

    /**
     * Initialize infobject resource controller.
     *
     * @param type InfobjectRepositoryInterface $infobject
     *
     * @return null
     */
    public function __construct(InfobjectRepositoryInterface $infobject)
    {
        parent::__construct();
        $this->repository = $infobject;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Infobjects\Repositories\Criteria\InfobjectResourceCriteria::class);
    }

    /**
     * Display a list of infobject.
     *
     * @return Response
     */
    public function index(InfobjectRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Infobjects\Repositories\Presenter\InfobjectPresenter::class)
                ->$function();
        }

        $infobjects = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('infobjects::infobject.names'))
            ->view('infobjects::infobject.index', true)
            ->data(compact('infobjects', 'view'))
            ->output();
    }

    /**
     * Display infobject.
     *
     * @param Request $request
     * @param Model   $infobject
     *
     * @return Response
     */
    public function show(InfobjectRequest $request, Infobject $infobject)
    {

        if ($infobject->exists) {
            $view = 'infobjects::infobject.show';
        } else {
            $view = 'infobjects::infobject.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('infobjects::infobject.name'))
            ->data(compact('infobject'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new infobject.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InfobjectRequest $request)
    {

        $infobject = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('infobjects::infobject.name')) 
            ->view('infobjects::infobject.create', true) 
            ->data(compact('infobject'))
            ->output();
    }

    /**
     * Create new infobject.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InfobjectRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $infobject                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('infobjects::infobject.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('infobjects/infobject/' . $infobject->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/infobjects/infobject'))
                ->redirect();
        }

    }

    /**
     * Show infobject for editing.
     *
     * @param Request $request
     * @param Model   $infobject
     *
     * @return Response
     */
    public function edit(InfobjectRequest $request, Infobject $infobject)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('infobjects::infobject.name'))
            ->view('infobjects::infobject.edit', true)
            ->data(compact('infobject'))
            ->output();
    }

    /**
     * Update the infobject.
     *
     * @param Request $request
     * @param Model   $infobject
     *
     * @return Response
     */
    public function update(InfobjectRequest $request, Infobject $infobject)
    {
        try {
            $attributes = $request->all();

            $infobject->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('infobjects::infobject.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('infobjects/infobject/' . $infobject->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('infobjects/infobject/' . $infobject->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the infobject.
     *
     * @param Model   $infobject
     *
     * @return Response
     */
    public function destroy(InfobjectRequest $request, Infobject $infobject)
    {
        try {

            $infobject->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('infobjects::infobject.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('infobjects/infobject/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('infobjects/infobject/' . $infobject->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple infobject.
     *
     * @param Model   $infobject
     *
     * @return Response
     */
    public function delete(InfobjectRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('infobjects::infobject.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('infobjects/infobject'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/infobjects/infobject'))
                ->redirect();
        }

    }

    /**
     * Restore deleted infobjects.
     *
     * @param Model   $infobject
     *
     * @return Response
     */
    public function restore(InfobjectRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('infobjects::infobject.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/infobjects/infobject'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/infobjects/infobject/'))
                ->redirect();
        }

    }

}
