<?php

namespace Btravel\Infobjects\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Infobjects\Http\Requests\InfocategoryRequest;
use Btravel\Infobjects\Interfaces\InfocategoryRepositoryInterface;
use Btravel\Infobjects\Models\Infocategory;

/**
 * Resource controller class for infocategory.
 */
class InfocategoryResourceController extends BaseController
{

    /**
     * Initialize infocategory resource controller.
     *
     * @param type InfocategoryRepositoryInterface $infocategory
     *
     * @return null
     */
    public function __construct(InfocategoryRepositoryInterface $infocategory)
    {
        parent::__construct();
        $this->repository = $infocategory;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Infobjects\Repositories\Criteria\InfocategoryResourceCriteria::class);
    }

    /**
     * Display a list of infocategory.
     *
     * @return Response
     */
    public function index(InfocategoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Infobjects\Repositories\Presenter\InfocategoryPresenter::class)
                ->$function();
        }

        $infocategories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('infobjects::infocategory.names'))
            ->view('infobjects::infocategory.index', true)
            ->data(compact('infocategories', 'view'))
            ->output();
    }

    /**
     * Display infocategory.
     *
     * @param Request $request
     * @param Model   $infocategory
     *
     * @return Response
     */
    public function show(InfocategoryRequest $request, Infocategory $infocategory)
    {

        if ($infocategory->exists) {
            $view = 'infobjects::infocategory.show';
        } else {
            $view = 'infobjects::infocategory.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('infobjects::infocategory.name'))
            ->data(compact('infocategory'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new infocategory.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InfocategoryRequest $request)
    {

        $infocategory = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('infobjects::infocategory.name')) 
            ->view('infobjects::infocategory.create', true) 
            ->data(compact('infocategory'))
            ->output();
    }

    /**
     * Create new infocategory.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InfocategoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $infocategory                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('infobjects::infocategory.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('infobjects/infocategory/' . $infocategory->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/infobjects/infocategory'))
                ->redirect();
        }

    }

    /**
     * Show infocategory for editing.
     *
     * @param Request $request
     * @param Model   $infocategory
     *
     * @return Response
     */
    public function edit(InfocategoryRequest $request, Infocategory $infocategory)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('infobjects::infocategory.name'))
            ->view('infobjects::infocategory.edit', true)
            ->data(compact('infocategory'))
            ->output();
    }

    /**
     * Update the infocategory.
     *
     * @param Request $request
     * @param Model   $infocategory
     *
     * @return Response
     */
    public function update(InfocategoryRequest $request, Infocategory $infocategory)
    {
        try {
            $attributes = $request->all();

            $infocategory->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('infobjects::infocategory.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('infobjects/infocategory/' . $infocategory->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('infobjects/infocategory/' . $infocategory->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the infocategory.
     *
     * @param Model   $infocategory
     *
     * @return Response
     */
    public function destroy(InfocategoryRequest $request, Infocategory $infocategory)
    {
        try {

            $infocategory->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('infobjects::infocategory.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('infobjects/infocategory/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('infobjects/infocategory/' . $infocategory->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple infocategory.
     *
     * @param Model   $infocategory
     *
     * @return Response
     */
    public function delete(InfocategoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('infobjects::infocategory.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('infobjects/infocategory'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/infobjects/infocategory'))
                ->redirect();
        }

    }

    /**
     * Restore deleted infocategories.
     *
     * @param Model   $infocategory
     *
     * @return Response
     */
    public function restore(InfocategoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('infobjects::infocategory.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/infobjects/infocategory'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/infobjects/infocategory/'))
                ->redirect();
        }

    }

}
